@@ -60,6 +60,8 @@ bool CryptData::SetCryptKeys(bool Encrypt,CRYPT_METHOD Method,
   WideToChar(PwdW,PwdA,ASIZE(PwdA));
   PwdA[Min(MAXPASSWORD_RAR,MAXPASSWORD)-1]=0; // For compatibility with existing archives.
 
+  bool Success=true;
+  
   switch(Method)
   {
 #ifndef SFX_MODULE
@@ -77,12 +79,12 @@ bool CryptData::SetCryptKeys(bool Encrypt,CRYPT_METHOD Method,
       SetKey30(Encrypt,Password,PwdW,Salt);
       break;
     case CRYPT_RAR50:
-      SetKey50(Encrypt,Password,PwdW,Salt,InitV,Lg2Cnt,HashKey,PswCheck);
+      Success=SetKey50(Encrypt,Password,PwdW,Salt,InitV,Lg2Cnt,HashKey,PswCheck);
       break;
   }
   cleandata(PwdA,sizeof(PwdA));
   cleandata(PwdW,sizeof(PwdW));
-  return true;
+  return Success;
 }
 
 
