@@ -3,7 +3,8 @@
 
 
 enum CRYPT_METHOD {
-  CRYPT_NONE,CRYPT_RAR13,CRYPT_RAR15,CRYPT_RAR20,CRYPT_RAR30,CRYPT_RAR50
+  CRYPT_NONE,CRYPT_RAR13,CRYPT_RAR15,CRYPT_RAR20,CRYPT_RAR30,CRYPT_RAR50,
+  CRYPT_UNKNOWN
 };
 
 #define SIZE_SALT50              16
