@@ -19,19 +19,13 @@
 
 // RAR2 service header extra records.
 #ifndef SFX_MODULE
-void SetExtraInfo20(CommandData *Cmd,Archive &Arc,wchar *Name)
+void SetExtraInfo20(CommandData *Cmd,Archive &Arc,const std::wstring &Name)
 {
+#ifdef _WIN_ALL
   if (Cmd->Test)
     return;
   switch(Arc.SubBlockHead.SubType)
   {
-#ifdef _UNIX
-    case UO_HEAD:
-      if (Cmd->ProcessOwners)
-        ExtractUnixOwner20(Arc,Name);
-      break;
-#endif
-#ifdef _WIN_ALL
     case NTACL_HEAD:
       if (Cmd->ProcessOwners)
         ExtractACL20(Arc,Name);
@@ -39,19 +33,19 @@ void SetExtraInfo20(CommandData *Cmd,Archive &Arc,wchar *Name)
     case STREAM_HEAD:
       ExtractStreams20(Arc,Name);
       break;
-#endif
   }
+#endif
 }
 #endif
 
 
 // RAR3 and RAR5 service header extra records.
-void SetExtraInfo(CommandData *Cmd,Archive &Arc,wchar *Name)
+void SetExtraInfo(CommandData *Cmd,Archive &Arc,const std::wstring &Name)
 {
 #ifdef _UNIX
   if (!Cmd->Test && Cmd->ProcessOwners && Arc.Format==RARFMT15 &&
       Arc.SubHead.CmpName(SUBHEAD_TYPE_UOWNER))
-    ExtractUnixOwner30(Arc,Name);
+    ExtractUnixOwner30(Arc,Name.c_str());
 #endif
 #ifdef _WIN_ALL
   if (!Cmd->Test && Cmd->ProcessOwners && Arc.SubHead.CmpName(SUBHEAD_TYPE_ACL))
@@ -63,7 +57,7 @@ void SetExtraInfo(CommandData *Cmd,Archive &Arc,wchar *Name)
 
 
 // Extra data stored directly in file header.
-void SetFileHeaderExtra(CommandData *Cmd,Archive &Arc,wchar *Name)
+void SetFileHeaderExtra(CommandData *Cmd,Archive &Arc,const std::wstring &Name)
 {
 #ifdef _UNIX
    if (Cmd->ProcessOwners && Arc.Format==RARFMT50 && Arc.FileHead.UnixOwnerSet)
@@ -74,36 +68,34 @@ void SetFileHeaderExtra(CommandData *Cmd,Archive &Arc,wchar *Name)
 
 
 
-// Calculate a number of path components except \. and \..
-static int CalcAllowedDepth(const wchar *Name)
+// Calculate the number of path components except \. and \..
+static int CalcAllowedDepth(const std::wstring &Name)
 {
   int AllowedDepth=0;
-  while (*Name!=0)
-  {
-    if (IsPathDiv(Name[0]) && Name[1]!=0 && !IsPathDiv(Name[1]))
+  for (size_t I=0;I<Name.size();I++)
+    if (IsPathDiv(Name[I]))
     {
-      bool Dot=Name[1]=='.' && (IsPathDiv(Name[2]) || Name[2]==0);
-      bool Dot2=Name[1]=='.' && Name[2]=='.' && (IsPathDiv(Name[3]) || Name[3]==0);
+      bool Dot=Name[I+1]=='.' && (IsPathDiv(Name[I+2]) || Name[I+2]==0);
+      bool Dot2=Name[I+1]=='.' && Name[I+2]=='.' && (IsPathDiv(Name[I+3]) || Name[I+3]==0);
       if (!Dot && !Dot2)
         AllowedDepth++;
+      else
+        if (Dot2)
+          AllowedDepth--;
     }
-    Name++;
-  }
-  return AllowedDepth;
+  return AllowedDepth < 0 ? 0 : AllowedDepth;
 }
 
 
 // Check if all existing path components are directories and not links.
-static bool LinkInPath(const wchar *Name)
+static bool LinkInPath(std::wstring Path)
 {
-  wchar Path[NM];
-  if (wcslen(Name)>=ASIZE(Path))
-    return true;  // It should not be that long, skip.
-  wcsncpyz(Path,Name,ASIZE(Path));
-  for (wchar *s=Path+wcslen(Path)-1;s>Path;s--)
-    if (IsPathDiv(*s))
+  if (Path.empty()) // So we can safely use Path.size()-1 below.
+    return false;
+  for (size_t I=Path.size()-1;I>0;I--)
+    if (IsPathDiv(Path[I]))
     {
-      *s=0;
+      Path.erase(I);
       FindData FD;
       if (FindFile::FastFind(Path,&FD,true) && (FD.IsLink || !FD.IsDir))
         return true;
@@ -112,69 +104,7 @@ static bool LinkInPath(const wchar *Name)
 }
 
 
-// Delete symbolic links in file path, if any, and replace them by directories.
-// Prevents extracting files outside of destination folder with symlink chains.
-bool LinksToDirs(const wchar *SrcName,const wchar *SkipPart,std::wstring &LastChecked)
-{
-  // Unlike Unix, Windows doesn't expand lnk1 in symlink targets like
-  // "lnk1/../dir", but converts the path to "dir". In Unix we need to call
-  // this function to prevent placing unpacked files outside of destination
-  // folder if previously we unpacked "dir/lnk1" -> "..",
-  // "dir/lnk2" -> "lnk1/.." and "dir/lnk2/anypath/poc.txt".
-  // We may still need this function to prevent abusing symlink chains
-  // in link source path if we remove detection of such chains
-  // in IsRelativeSymlinkSafe. This function seems to make other symlink
-  // related safety checks redundant, but for now we prefer to keep them too.
-  //
-  // 2022.12.01: the performance impact is minimized after adding the check
-  // against the previous path and enabling this verification only after
-  // extracting a symlink with ".." in target. So we enabled it for Windows
-  // as well for extra safety.
-//#ifdef _UNIX
-  wchar Path[NM];
-  if (wcslen(SrcName)>=ASIZE(Path))
-    return false;  // It should not be that long, skip.
-  wcsncpyz(Path,SrcName,ASIZE(Path));
-
-  size_t SkipLength=wcslen(SkipPart);
-
-  if (SkipLength>0 && wcsncmp(Path,SkipPart,SkipLength)!=0)
-    SkipLength=0; // Parameter validation, not really needed now.
-
-  // Do not check parts already checked in previous path to improve performance.
-  for (uint I=0;Path[I]!=0 && I<LastChecked.size() && Path[I]==LastChecked[I];I++)
-    if (IsPathDiv(Path[I]) && I>SkipLength)
-      SkipLength=I;
-
-  wchar *Name=Path;
-  if (SkipLength>0)
-  {
-    // Avoid converting symlinks in destination path part specified by user.
-    Name+=SkipLength;
-    while (IsPathDiv(*Name))
-      Name++;
-  }
-
-  for (wchar *s=Path+wcslen(Path)-1;s>Name;s--)
-    if (IsPathDiv(*s))
-    {
-      *s=0;
-      FindData FD;
-      if (FindFile::FastFind(Path,&FD,true) && FD.IsLink)
-#ifdef _WIN_ALL
-        if (!DelDir(Path))
-#else
-        if (!DelFile(Path))
-#endif
-          return false; // Couldn't delete the symlink to replace it with directory.
-    }
-  LastChecked=SrcName;
-//#endif
-  return true;
-}
-
-
-bool IsRelativeSymlinkSafe(CommandData *Cmd,const wchar *SrcName,const wchar *PrepSrcName,const wchar *TargetName)
+bool IsRelativeSymlinkSafe(CommandData *Cmd,const std::wstring &SrcName,std::wstring PrepSrcName,const std::wstring &TargetName)
 {
   // Catch root dir based /path/file paths also as stuff like \\?\.
   // Do not check PrepSrcName here, it can be root based if destination path
@@ -184,14 +114,13 @@ bool IsRelativeSymlinkSafe(CommandData *Cmd,const wchar *SrcName,const wchar *Pr
 
   // Number of ".." in link target.
   int UpLevels=0;
-  for (int Pos=0;*TargetName!=0;Pos++)
+  for (uint Pos=0;Pos<TargetName.size();Pos++)
   {
-    bool Dot2=TargetName[0]=='.' && TargetName[1]=='.' && 
-              (IsPathDiv(TargetName[2]) || TargetName[2]==0) &&
-              (Pos==0 || IsPathDiv(*(TargetName-1)));
+    bool Dot2=TargetName[Pos]=='.' && TargetName[Pos+1]=='.' && 
+              (IsPathDiv(TargetName[Pos+2]) || TargetName[Pos+2]==0) &&
+              (Pos==0 || IsPathDiv(TargetName[Pos-1]));
     if (Dot2)
       UpLevels++;
-    TargetName++;
   }
   // If link target includes "..", it must not have another links in its
   // source path, because they can bypass our safety check. For example,
@@ -213,12 +142,12 @@ bool IsRelativeSymlinkSafe(CommandData *Cmd,const wchar *SrcName,const wchar *Pr
   // Remove the destination path from prepared name if any. We should not
   // count the destination path depth, because the link target must point
   // inside of this path, not outside of it.
-  size_t ExtrPathLength=wcslen(Cmd->ExtrPath);
-  if (ExtrPathLength>0 && wcsncmp(PrepSrcName,Cmd->ExtrPath,ExtrPathLength)==0)
+  size_t ExtrPathLength=Cmd->ExtrPath.size();
+  if (ExtrPathLength>0 && PrepSrcName.compare(0,ExtrPathLength,Cmd->ExtrPath)==0)
   {
-    PrepSrcName+=ExtrPathLength;
-    while (IsPathDiv(*PrepSrcName))
-      PrepSrcName++;
+    while (IsPathDiv(PrepSrcName[ExtrPathLength]))
+      ExtrPathLength++;
+    PrepSrcName.erase(0,ExtrPathLength);
   }
   int PrepAllowedDepth=CalcAllowedDepth(PrepSrcName);
 
@@ -226,7 +155,7 @@ bool IsRelativeSymlinkSafe(CommandData *Cmd,const wchar *SrcName,const wchar *Pr
 }
 
 
-bool ExtractSymlink(CommandData *Cmd,ComprDataIO &DataIO,Archive &Arc,const wchar *LinkName,bool &UpLink)
+bool ExtractSymlink(CommandData *Cmd,ComprDataIO &DataIO,Archive &Arc,const std::wstring &LinkName,bool &UpLink)
 {
   // Returning true in Uplink indicates that link target might include ".."
   // and enables additional checks. It is ok to falsely return true here,
@@ -236,20 +165,20 @@ bool ExtractSymlink(CommandData *Cmd,ComprDataIO &DataIO,Archive &Arc,const wcha
   UpLink=true; // Assume the target might include potentially unsafe "..".
 #if defined(SAVE_LINKS) && defined(_UNIX) || defined(_WIN_ALL)
   if (Arc.Format==RARFMT50) // For RAR5 archives we can check RedirName for both Unix and Windows.
-    UpLink=wcsstr(Arc.FileHead.RedirName,L"..")!=NULL;
+    UpLink=Arc.FileHead.RedirName.find(L"..")!=std::wstring::npos;
 #endif
 
 #if defined(SAVE_LINKS) && defined(_UNIX)
   // For RAR 3.x archives we process links even in test mode to skip link data.
   if (Arc.Format==RARFMT15)
-    return ExtractUnixLink30(Cmd,DataIO,Arc,LinkName,UpLink);
+    return ExtractUnixLink30(Cmd,DataIO,Arc,LinkName.c_str(),UpLink);
   if (Arc.Format==RARFMT50)
-    return ExtractUnixLink50(Cmd,LinkName,&Arc.FileHead);
+    return ExtractUnixLink50(Cmd,LinkName.c_str(),&Arc.FileHead);
 #elif defined(_WIN_ALL)
   // RAR 5.0 archives store link information in file header, so there is
   // no need to additionally test it if we do not create a file.
   if (Arc.Format==RARFMT50)
-    return CreateReparsePoint(Cmd,LinkName,&Arc.FileHead);
+    return CreateReparsePoint(Cmd,LinkName.c_str(),&Arc.FileHead);
 #endif
   return false;
 }
