@@ -97,6 +97,7 @@
 #define   MCHelpSwMC         L"\n  mc<par>       Set advanced compression parameters"
 #define   MCHelpSwMD         L"\n  md[x]<n>[kmg] Dictionary size in KB, MB or GB"
 #define   MCHelpSwME         L"\n  me[par]       Set encryption parameters"
+#define   MCHelpSwMLP        L"\n  mlp           Use large memory pages"
 #define   MCHelpSwMS         L"\n  ms[ext;ext]   Specify file types to store"
 #define   MCHelpSwMT         L"\n  mt<threads>   Set the number of threads"
 #define   MCHelpSwN          L"\n  n<file>       Additionally filter included files"
@@ -107,6 +108,7 @@
 #define   MCHelpSwOH         L"\n  oh            Save hard links as the link instead of the file"
 #define   MCHelpSwOI         L"\n  oi[0-4][:min] Save identical files as references"
 #define   MCHelpSwOL         L"\n  ol[a,-]       Process symbolic links as the link [absolute paths, skip]"
+#define   MCHelpSwOM         L"\n  om[-|1][=lst] Propagate Mark of the Web"
 #define   MCHelpSwONI        L"\n  oni           Allow potentially incompatible names"
 #define   MCHelpSwOP         L"\n  op<path>      Set the output path for extracted files"
 #define   MCHelpSwOR         L"\n  or            Rename files automatically"
@@ -232,8 +234,8 @@
 #define   MArcComment        L"\nArchive comment"
 #define   MReadStdinCmt      L"\nReading comment from stdin\n"
 #define   MReadCommFrom      L"\nReading comment from %s"
-#define   MDelComment        L"\nDeleting comment from %s"
-#define   MAddComment        L"\nAdding comment to %s"
+#define   MDelComment        L"\nDeleting a comment from %s"
+#define   MAddComment        L"\nAdding a comment to %s"
 #define   MLogCommBrk        L"\nThe archive comment is corrupt"
 #define   MCommAskCont       L"\nPress 'Enter' to continue or 'Q' to quit:"
 #define   MWriteCommTo       L"\nWrite comment to %s"
@@ -302,7 +304,7 @@
 #define   MRecSecDamage      L"\nSector %ld (offsets %lX...%lX) damaged"
 #define   MRecCorrected      L" - data recovered"
 #define   MRecFailed         L" - cannot recover data"
-#define   MAddRecRec         L"\nAdding data recovery record"
+#define   MAddRecRec         L"\nAdding the data recovery record"
 #define   MEraseForVolume    L"\n\nErasing contents of drive %c:\n"
 #define   MGetOwnersError    L"\nWARNING: Cannot get %s owner and group\n"
 #define   MErrGetOwnerID     L"\nWARNING: Cannot get owner %s ID\n"
@@ -329,7 +331,6 @@
 #define   MSubHeadCorrupt    L"\nERROR: Corrupt data header found, ignored"
 #define   MSubHeadUnknown    L"\nWARNING: Unknown data header format, ignored"
 #define   MSubHeadDataCRC    L"\nERROR: Corrupt %s data block"
-#define   MSubHeadType       L"\nData header type: %s"
 #define   MScanError         L"\nCannot read contents of %s"
 #define   MNotVolume         L"\n%s is not volume"
 #define   MRecVolDiffSets    L"\nERROR: %s and %s belong to different sets"
@@ -384,7 +385,7 @@
 #define   MUseSmalllerDict   L"\nPlease use a smaller compression dictionary."
 #define   MExtrDictOutMem    L"\nNot enough memory to unpack the archive with %u GB compression dictionary."
 #define   MSuggest64bit      L"\n64-bit RAR version is necessary."
-#define   MOpenErrAtime      L"\nYou may need to remove -tsp switch to open this file."
+#define   MOpenErrAtime      L"\nYou may need to remove -tsp switch or run RAR as administrator to open this file."
 #define   MErrReadInfo       L"\nChoose 'Ignore' to continue with the already read file part only, 'Ignore all' to do it for all read errors, 'Retry' to repeat read and 'Quit' to abort."
 #define   MErrReadTrunc      L"\n%s is archived incompletely because of read error.\n"
 #define   MErrReadCount      L"\n%u files are archived incompletely because of read errors."
@@ -402,3 +403,8 @@
 #define   MDictComprLimit    L"\n%u GB dictionary exceeds %u GB limit and not allowed when compressing data."
 #define   MNeedSFX64         L"\n64-bit self-extracting module is necessary for %u GB compression dictionary."
 #define   MSkipUnsafeLink    L"\nSkipping the potentially unsafe %s -> %s link. For archives from a trustworthy source use -ola to extract it anyway."
+#define   MTruncService      L"\nTruncated at the service block: %s"
+#define   MHeaderQO          L"quick open information"
+#define   MHeaderRR          L"recovery record"
+#define   MLockInMemoryNeeded L"-mlp switch requires ""Lock pages in memory"" privilege. Do you wish to assign it to the current user account?"
+#define   MPrivilegeAssigned L"User privilege has been successfully assigned and will be activated after Windows restart. Restart now?"
